@@ -5,7 +5,10 @@ from ccxt.base.exchange import Exchange
 from ccxt.base.errors import ExchangeError
 from ccxt.base.errors import AuthenticationError
 from ccxt.base.errors import PermissionDenied
+from ccxt.base.errors import AccountSuspended
 from ccxt.base.errors import ArgumentsRequired
+from ccxt.base.errors import BadRequest
+from ccxt.base.errors import BadSymbol
 from ccxt.base.errors import InsufficientFunds
 from ccxt.base.errors import InvalidAddress
 from ccxt.base.errors import InvalidOrder
@@ -86,6 +89,7 @@ class binance(Exchange):
                     'sapi': 'https://api.binance.com/sapi/v1',
                     'fapiPublic': 'https://fapi.binance.com/fapi/v1',
                     'fapiPrivate': 'https://fapi.binance.com/fapi/v1',
+                    'fapiData': 'https://fapi.binance.com/futures/data',
                     'fapiPrivateV2': 'https://fapi.binance.com/fapi/v2',
                     'dapiPublic': 'https://dapi.binance.com/dapi/v1',
                     'dapiPrivate': 'https://dapi.binance.com/dapi/v1',
@@ -127,6 +131,10 @@ class binance(Exchange):
                         'margin/myTrades',
                         'margin/maxBorrowable',
                         'margin/maxTransferable',
+                        'margin/isolated/transfer',
+                        'margin/isolated/account',
+                        'margin/isolated/pair',
+                        'margin/isolated/allPairs',
                         'futures/transfer',
                         # https://binance-docs.github.io/apidocs/spot/en/#withdraw-sapi
                         'capital/config/getall',  # get networks for withdrawing USDT ERC20 vs USDT Omni
@@ -141,6 +149,7 @@ class binance(Exchange):
                         'sub-account/margin/account',
                         'sub-account/margin/accountSummary',
                         'sub-account/status',
+                        'sub-account/transfer/subUserHistory',
                         # lending endpoints
                         'lending/daily/product/list',
                         'lending/daily/userLeftQuota',
@@ -170,10 +179,13 @@ class binance(Exchange):
                         'margin/loan',
                         'margin/repay',
                         'margin/order',
+                        'margin/isolated/create',
+                        'margin/isolated/transfer',
                         'sub-account/margin/enable',
                         'sub-account/margin/enable',
                         'sub-account/futures/enable',
                         'userDataStream',
+                        'userDataStream/isolated',
                         'futures/transfer',
                         # lending
                         'lending/customizedFixed/purchase',
@@ -182,10 +194,12 @@ class binance(Exchange):
                     ],
                     'put': [
                         'userDataStream',
+                        'userDataStream/isolated',
                     ],
                     'delete': [
                         'margin/order',
                         'userDataStream',
+                        'userDataStream/isolated',
                     ],
                 },
                 'wapi': {
@@ -225,7 +239,17 @@ class binance(Exchange):
                         'ticker/bookTicker',
                         'allForceOrders',
                         'openInterest',
-                        'leverageBracket',
+                    ],
+                },
+                'fapiData': {
+                    'get': [
+                        'openInterestHist',
+                        'topLongShortAccountRatio',
+                        'topLongShortPositionRatio',
+                        'globalLongShortAccountRatio',
+                        'takerlongshortRatio',
+                        'takerBuySellVol',
+                        'basis'
                     ],
                 },
                 'fapiPrivate': {
@@ -241,6 +265,9 @@ class binance(Exchange):
                         'positionSide/dual',
                         'userTrades',
                         'income',
+                        'leverageBracket',
+                        'adlQuantile',
+                        'forceOrders',
                     ],
                     'post': [
                         'batchOrders',
@@ -280,6 +307,7 @@ class binance(Exchange):
                         'historicalTrades',
                         'aggTrades',
                         'premiumIndex',
+                        'fundingRate',
                         'klines',
                         'continuousKlines',
                         'indexPriceKlines',
@@ -298,12 +326,15 @@ class binance(Exchange):
                         'openOrder',
                         'openOrders',
                         'allOrders',
+                        'balance',
                         'account',
                         'positionMargin/history',
                         'positionRisk',
                         'userTrades',
                         'income',
                         'leverageBracket',
+                        'forceOrders',
+                        'adlQuantile',
                     ],
                     'post': [
                         'positionSide/dual',
@@ -390,7 +421,7 @@ class binance(Exchange):
                 'fetchTradesMethod': 'publicGetAggTrades',  # publicGetTrades, publicGetHistoricalTrades
                 'fetchTickersMethod': 'publicGetTicker24hr',
                 'defaultTimeInForce': 'GTC',  # 'GTC' = Good To Cancel(default), 'IOC' = Immediate Or Cancel
-                'defaultType': 'spot',  # 'spot', 'futures', 'swap', 'margin'
+                'defaultType': 'spot',  # 'spot', 'margin', 'futures', 'swap', 'swap.usdt'
                 'hasAlreadyAuthenticatedSuccessfully': False,
                 'warnOnFetchOpenOrdersWithoutSymbol': True,
                 'recvWindow': 5 * 1000,  # 5 sec, binance default
@@ -411,20 +442,46 @@ class binance(Exchange):
                 "You don't have permission.": PermissionDenied,  # {"msg":"You don't have permission.","success":false}
                 'Market is closed.': ExchangeNotAvailable,  # {"code":-1013,"msg":"Market is closed."}
                 '-1000': ExchangeNotAvailable,  # {"code":-1000,"msg":"An unknown error occured while processing the request."}
+                '-1001': ExchangeNotAvailable,  # 'Internal error; unable to process your request. Please try again.'
+                '-1002': AuthenticationError,  # 'You are not authorized to execute self request.'
                 '-1003': RateLimitExceeded,  # {"code":-1003,"msg":"Too much request weight used, current limit is 1200 request weight per 1 MINUTE. Please use the websocket for live updates to avoid polling the API."}
                 '-1013': InvalidOrder,  # createOrder -> 'invalid quantity'/'invalid price'/MIN_NOTIONAL
+                '-1015': RateLimitExceeded,  # 'Too many new orders; current limit is %s orders per %s.'
+                '-1016': ExchangeNotAvailable,  # 'This service is no longer available.',
+                '-1020': BadRequest,  # 'This operation is not supported.'
                 '-1021': InvalidNonce,  # 'your time is ahead of server'
                 '-1022': AuthenticationError,  # {"code":-1022,"msg":"Signature for self request is not valid."}
-                '-1100': InvalidOrder,  # createOrder(symbol, 1, asdf) -> 'Illegal characters found in parameter 'price'
-                '-1104': ExchangeError,  # Not all sent parameters were read, read 8 parameters but was sent 9
-                '-1128': ExchangeError,  # {"code":-1128,"msg":"Combination of optional parameters invalid."}
+                '-1100': BadRequest,  # createOrder(symbol, 1, asdf) -> 'Illegal characters found in parameter 'price'
+                '-1101': BadRequest,  # Too many parameters; expected %s and received %s.
+                '-1102': BadRequest,  # Param %s or %s must be sent, but both were empty
+                '-1103': BadRequest,  # An unknown parameter was sent.
+                '-1104': BadRequest,  # Not all sent parameters were read, read 8 parameters but was sent 9
+                '-1105': BadRequest,  # Parameter %s was empty.
+                '-1106': BadRequest,  # Parameter %s sent when not required.
+                '-1111': BadRequest,  # Precision is over the maximum defined for self asset.
+                '-1112': InvalidOrder,  # No orders on book for symbol.
+                '-1114': BadRequest,  # TimeInForce parameter sent when not required.
+                '-1115': BadRequest,  # Invalid timeInForce.
+                '-1116': BadRequest,  # Invalid orderType.
+                '-1117': BadRequest,  # Invalid side.
+                '-1118': BadRequest,  # New client order ID was empty.
+                '-1119': BadRequest,  # Original client order ID was empty.
+                '-1120': BadRequest,  # Invalid interval.
+                '-1121': BadSymbol,  # Invalid symbol.
+                '-1125': AuthenticationError,  # This listenKey does not exist.
+                '-1127': BadRequest,  # More than %s hours between startTime and endTime.
+                '-1128': BadRequest,  # {"code":-1128,"msg":"Combination of optional parameters invalid."}
+                '-1130': BadRequest,  # Data sent for paramter %s is not valid.
+                '-1131': BadRequest,  # recvWindow must be less than 60000
                 '-2010': ExchangeError,  # generic error code for createOrder -> 'Account has insufficient balance for requested action.', {"code":-2010,"msg":"Rest API trading is not enabled."}, etc...
                 '-2011': OrderNotFound,  # cancelOrder(1, 'BTC/USDT') -> 'UNKNOWN_ORDER'
                 '-2013': OrderNotFound,  # fetchOrder(1, 'BTC/USDT') -> 'Order does not exist'
                 '-2014': AuthenticationError,  # {"code":-2014, "msg": "API-key format invalid."}
                 '-2015': AuthenticationError,  # "Invalid API-key, IP, or permissions for action."
+                '-3005': InsufficientFunds,  # {"code":-3005,"msg":"Transferring out not allowed. Transfer out amount exceeds max amount."}
                 '-3008': InsufficientFunds,  # {"code":-3008,"msg":"Borrow not allowed. Your borrow amount has exceed maximum borrow amount."}
                 '-3010': ExchangeError,  # {"code":-3010,"msg":"Repay not allowed. Repay amount exceeds borrow amount."}
+                '-3022': AccountSuspended,  # You account's trading is banned.
             },
         })
 
@@ -432,10 +489,11 @@ class binance(Exchange):
         return self.milliseconds() - self.options['timeDifference']
 
     def method_by_type(self, method, type):
-        if type == 'futures':
-            prefix = 'dapi'
-        elif type == 'swap':
-            prefix = 'fapi'
+        if type.startswith(('futures', 'swap')):
+            if type.endswith('.usdt'):
+                prefix = 'fapi'
+            else:
+                prefix = 'dapi'
         else:
             prefix = ''
 
@@ -459,10 +517,10 @@ class binance(Exchange):
         defaultType = self.safe_string_2(self.options, 'fetchMarkets', 'defaultType', 'spot')
         type = self.safe_string(params, 'type', defaultType)
         query = self.omit(params, 'type')
-        if type not in ('spot', 'futures', 'swap', 'margin'):
+        if type not in ('spot', 'futures', 'swap', 'swap.usdt', 'margin'):
             raise ExchangeError(
                 f"{self.id} does not support '{type}' type, set exchange.options['defaultType'] "
-                f"to 'spot', 'futures', 'swap', 'margin'")
+                f"to 'spot', 'futures', 'swap', 'swap.usdt', 'margin'")
         method = self.method_by_type('publicGetExchangeInfo', type)
         response = getattr(self, method)(query)
         #
@@ -550,12 +608,16 @@ class binance(Exchange):
             spot = futures = swap = False
             marketType = None
             if 'maintMarginPercent' in market:
-                if 'deliveryDate' in market:
+                contract_type = market.get('contractType')
+                if contract_type == 'PERPETUAL' or contract_type is None:
+                    swap = True
+                    if market['quoteAsset'] == 'USDT':
+                        marketType = 'swap.usdt'
+                    else:
+                        marketType = 'swap'
+                else:
                     futures = True
                     marketType = 'futures'
-                else:
-                    swap = True
-                    marketType = 'swap'
             else:
                 spot = True
                 marketType = 'spot'
@@ -565,7 +627,10 @@ class binance(Exchange):
             quoteId = self.safe_string(market, 'quoteAsset')
             base = self.safe_currency_code(baseId)
             quote = self.safe_currency_code(quoteId)
-            symbol = id if marketType == 'futures' else f'{base}/{quote}'
+            if marketType.startswith(('futures', 'swap')):
+                symbol = id
+            else:
+                symbol = f'{base}/{quote}'
             filters = self.safe_value(market, 'filters', [])
             filtersByType = self.index_by(filters, 'filterType')
             precision = {
